<?php
/**
 * @file
 * Administration pages for managing module settings.
 */

/**
 * Form to build settings admin screen.
 *
 * @see node_promo_picker_settings_admin_form_validate()
 * @see node_promo_picker_settings_admin_form_submit()
 */
function node_promo_picker_settings_admin_form($form, &$state) {
  $form = array();
  // Add settings.
  $form['settings_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Settings',
  );
  $theme_path = drupal_get_path('theme', variable_get('theme_default', 'garland'));
  $module_path = drupal_get_path('module', 'node_promo_picker');
  $form['settings_panel']['css_path'] = array(
    '#type' => 'textfield',
    '#title' => 'CSS file',
    '#description' => filter_xss('Define the path to the CSS file to be included. <div>%t - path to theme (' . $theme_path . ')</div><div>%m - path to module (' . $module_path . ')</div><div>Leave blank to not include a css file.</div>', array('div')),
    '#default_value' => variable_get('promo_picker_css_file', ''),
  );
  $form['settings_panel']['disable_blocks'] = array(
    '#type' => 'checkbox',
    '#title' => 'Disable blocks',
    '#description' => filter_xss('Check this to remove the creation of blocks for every term associated with a promo block.'),
    '#default_value' => variable_get('promo_picker_disable_block', 0),
  );
  foreach (array('primary', 'secondary') as $block) {
    $form['settings_panel'][$block] = array(
      '#type' => 'fieldset',
      '#title' => check_plain(ucwords($block) . ' block'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['settings_panel'][$block][$block . '_template_path'] = array(
      '#type' => 'textfield',
      '#title' => 'Template file',
      '#description' => filter_xss('Define the path to the template file to format the ' . $block . ' promo block. <div>%t - path to theme (' . $theme_path . ')</div><div>%m - path to module (' . $module_path . ')</div>', array('div')),
      '#default_value' => variable_get('promo_picker_template_file_' . $block, '%m/node_promo_picker_' . $block . '.tpl.php'),
    );
    $form['settings_panel'][$block][$block . '_max_teasers'] = array(
      '#type' => 'select',
      '#title' => check_plain('Maximum number of teasers'),
      '#description' => check_plain('Define the maximum number of teasers for the ' . $block . ' promo block.'),
      '#options' => array(1 => 1, 2 => 2, 3 => 3, 4 => 4, 5 => 5, 6 => 6, 7 => 7, 8 => 8, 9 => 9, 10 => 10),
      '#default_value' => variable_get('promo_picker_max_' . $block . '_teasers', 3),
    );
  }
  $form['settings_panel']['additional_themes'] = array(
    '#type' => 'fieldset',
    '#title' => 'Additional themes',
    '#description' => filter_xss('<div>Add additional theme templates to display block content.</div>Template file placeholders. <div>%t - path to theme (' . $theme_path . ')</div><div>%m - path to module (' . $module_path . ')</div>', array('div')),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['settings_panel']['additional_themes']['themes'] = array(
    '#prefix' => '<div id="additional-themes">',
    '#suffix' => '</div>',
    '#theme' => 'form_additional_themes',
    '#tree' => TRUE,
  );
  // Get the additional themes...
  $themes = variable_get('promo_picker_additional_themes', array());
  // ...and count them...
  if (empty($state['num_additional_themes'])) {
    $state['num_additional_themes'] = count( $themes ) + 1; // Add one for a blank row at the end of the form.
  }
  // ...and add a row to the form for each one.
  for ($i = 0; $i < $state['num_additional_themes']; $i++) {
    $form['settings_panel']['additional_themes']['themes'][$i] = array(
      'title' => array(
        '#type' => 'textfield',
        '#title' => '',
        '#maxlength' => 128,
        '#size' => 40,
        '#default_value' => '',
      ),
      'file' => array(
        '#type' => 'textfield',
        '#title' => '',
        '#size' => 40,
        '#default_value' => '',
      ),
    );
  }
  // Now populate all but the last fields with the existing theme values.
  $counter = 0;
  foreach ($themes as $theme) {
    $form['settings_panel']['additional_themes']['themes'][$counter]['title']['#default_value'] = $theme['title'];
    $form['settings_panel']['additional_themes']['themes'][$counter]['file']['#default_value'] = $theme['file'];
    $counter++;
  }
  $form['settings_panel']['additional_themes']['add_theme'] = array(
    '#type' => 'submit',
    '#value' => t('Add another theme'),
    '#submit' => array('node_promo_picker_ajax_add_more_theme_fields_add_one'),
    '#ajax' => array(
      'callback' => 'node_promo_picker_ajax_add_more_theme_fields_callback',
      'wrapper' => 'additional-themes',
    ),
  );
  // Add taxonomy settings.
  $form['taxonomy_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Taxonomy settings',
    '#description' => 'Associated vocabularies to promo teasers.',
  );
  // Get vocabulary records.
  $options = array();
  $t_settings = variable_get('promo_picker_taxonomy_settings', array());
  $vocabularies = db_query('SELECT name, vid FROM {taxonomy_vocabulary} ORDER BY name ASC');
  foreach ($vocabularies as $v) {
    $options[$v->vid] = $v->name;
  }
  $form['taxonomy_panel']['vocabularies'] = array(
    '#type' => 'checkboxes',
    '#title' => 'Vocabularies',
    '#options' => $options,
    '#default_value' => $t_settings,
  );
  // Add node settings fields.
  $form['node_type_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Node type settings',
  );
  // Get node type records.
  $types = db_query('SELECT name, type FROM {node_type} ORDER BY name ASC');
  $noptions = array();
  foreach ($types as $t) {
    $noptions[$t->type] = $t->name;
  }
  $form['node_type_panel']['image_settings'] = array(
    '#type' => 'fieldset',
    '#title' => 'Image settings',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['node_type_panel']['image_settings']['image_node_type'] = array(
    '#type' => 'select',
    '#title' => 'Image node type',
    '#description' => 'Define the node type to use for images associated to the teaser record.',
    '#options' => array_merge(array('' => 'NONE'), $noptions),
    '#default_value' => variable_get('promo_picker_image_node', ''),
  );
  $form['node_type_panel']['image_settings']['max_images'] = array(
    '#type' => 'select',
    '#title' => 'Maximum number of images',
    '#description' => 'Define the maximum number of images per teaser record.',
    '#options' => array(
      0 => 'Unlimited',
      1 => 1,
      2 => 2,
      3 => 3,
      4 => 4,
      5 => 5,
    ),
    '#default_value' => variable_get('promo_picker_max_images', 3),
  );
  $form['node_type_panel']['node_settings'] = array(
    '#type' => 'fieldset',
    '#title' => 'Node settings',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['node_type_panel']['node_settings']['teaser_node_types'] = array(
    '#type' => 'checkboxes',
    '#title' => 'Associated teaser node types',
    '#description' => 'Define the node types to allow teaser records for.',
    '#options' => $noptions,
    '#default_value' => variable_get('promo_picker_node_settings', array()),
  );
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

/**
 * Validate settings form submission.
 *
 * @see node_promo_picker_settings_admin_form()
 */
function node_promo_picker_settings_admin_form_validate($form, &$state) {
  if (!empty($state['triggering_element']['#ajax'])) {
    return;
  }
  if (empty($state['values']['primary_template_path'])) {
    form_set_error('primary_template_path', 'Enter a path to the primary template file.');
  }
  if (empty($state['values']['secondary_template_path'])) {
    form_set_error('secondary_template_path', 'Enter a path to the secondary template file.');
  }
  // Validate taxonomy settings.
  $vids = array();
  foreach ($state['values']['vocabularies'] as $vid => $checked) {
    if ($checked) {
      $vids[$vid] = $vid;
    }
  }
  if (empty($vids)) {
    form_set_error('', 'Set vocabularies to apply to teasers.');
  }
  // Validate node settings.
  $nodes = array();
  foreach ($state['values']['teaser_node_types'] as $nt => $checked) {
    if ($checked) {
      $nodes[] = $nt;
    }
  }
  if (empty($nodes)) {
    form_set_error('teaser_node_types', 'Set node types to apply create teasers for.');
  }
}

/**
 * Process settings form submission.
 *
 * @see node_promo_picker_settings_admin_form()
 */
function node_promo_picker_settings_admin_form_submit($form, &$state) {
  if (!empty($state['triggering_element']['#ajax'])) {
    return;
  }
  // Get taxonomy settings.
  $vids = array();
  foreach ($state['values']['vocabularies'] as $vid => $checked) {
    if ($checked) {
      $vids[$vid] = $vid;
    }
  }
  // Get node settings.
  $nodes = array();
  foreach ($state['values']['teaser_node_types'] as $nt => $checked) {
    if ($checked) {
      $nodes[] = $nt;
    }
  }
  // Get additional theme settings.
  $themes = array();
  foreach ($state['values']['themes'] as $theme) {
    if (!empty($theme['title']) && !empty($theme['file'])) {
      $themes[node_promo_picker_map_theme_title($theme['title'])] = array(
        'title' => $theme['title'],
        'file' => $theme['file'],
      );
    }
  }
  // Set variables.
  variable_set('promo_picker_taxonomy_settings', $vids);
  variable_set('promo_picker_node_settings', $nodes);
  variable_set('promo_picker_disable_block', $state['values']['disable_blocks']);
  variable_set('promo_picker_image_node', $state['values']['image_node_type']);
  variable_set('promo_picker_image_field', (isset($state['values']['image_field']) ? $state['values']['image_field'] : ''));
  variable_set('promo_picker_max_primary_teasers', $state['values']['primary_max_teasers']);
  variable_set('promo_picker_max_secondary_teasers', $state['values']['secondary_max_teasers']);
  variable_set('promo_picker_max_images', $state['values']['max_images']);
  variable_set('promo_picker_css_file', $state['values']['css_path']);
  variable_set('promo_picker_template_file_primary', $state['values']['primary_template_path']);
  variable_set('promo_picker_template_file_secondary', $state['values']['secondary_template_path']);
  variable_set('promo_picker_additional_themes', $themes);
  // Rebuild theme registry to ensure template files are updated.
  drupal_theme_rebuild();
  drupal_set_message(t('Promo picker settings saved.'));
  // Clear form cache.
  cache_clear_all('form_' . $state['values']['form_build_id'], 'cache_form');
  cache_clear_all('storage_' . $state['values']['form_build_id'], 'cache_form');
}

/**
 * Theme the additional themes settings field.
 */
function theme_form_additional_themes($element) {
  $element = $element['element'];
  $rows = array();
  $header = array('Theme title', 'Theme file');
  foreach (element_children($element) as $key) {
    if (isset($element[$key]['title'])) {
      $field = &$element[$key];
      $row = array();
      $row[] = drupal_render($field['title']);
      $row[] = drupal_render($field['file']);
      $rows[] = $row;
    }
  }
  if (empty($rows)) {
    $rows[] = array(array('data' => t('No additional themes.'), 'colspan' => '2'));
  }
  return theme('table', array('header' => $header, 'rows' => $rows));
}


/**
 * AJAX callback function for adding another row of theme fields to the form.
 */
function node_promo_picker_ajax_add_more_theme_fields_callback($form, $form_state) {
  return $form['settings_panel']['additional_themes']['themes'];
}

/**
 * Forces the form to rebuild itself, adding one more row of theme fields.
 */
function node_promo_picker_ajax_add_more_theme_fields_add_one($form, &$form_state) {
  $form_state['num_additional_themes']++;
  $form_state['rebuild'] = TRUE;
}
